// ====================================================================
// Copyright (c) 1997, 1998 The Apache Group.  All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer. 
//
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//
// 3. All advertising materials mentioning features or use of this
//    software must display the following acknowledgment:
//    "This product includes software developed by the Apache Group
//    for use in the Apache HTTP server project (http://www.apache.org/)."
//
// 4. The names "Apache Server" and "Apache Group" must not be used to
//    endorse or promote products derived from this software without
//    prior written permission.
//
// 5. Redistributions of any form whatsoever must retain the following
//    acknowledgment:
//    "This product includes software developed by the Apache Group
//    for use in the Apache HTTP server project (http://www.apache.org/)."
//
// THIS SOFTWARE IS PROVIDED BY THE APACHE GROUP ``AS IS'' AND ANY
// EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE APACHE GROUP OR
// ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
// OF THE POSSIBILITY OF SUCH DAMAGE.
// ====================================================================
//
// This software consists of voluntary contributions made by many
// individuals on behalf of the Apache Group and was originally based
// on public domain software written at the National Center for
// Supercomputing Applications, University of Illinois, Urbana-Champaign.
// For more information on the Apache Group and the Apache HTTP server
// project, please see <http://www.apache.org/>.

// JServ - Serve up Java servlets
// by Alexei Kosut <akosut@apache.org>

// Parts are based on examples from  _Java in a Nutshell_ by David Flanagan:
// Written by David Flanagan.  Copyright (c) 1996 O'Reilly & Associates.
// You may study, use, modify, and distribute this example for any purpose.
// This example is provided WITHOUT WARRANTY either expressed or implied.

// ServletMonitor.java:
// - org.apache.jserv.test.ServletMonitor

package gs.html;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

/**
 * Servlets that list the servlets in this virtual host
 * and their init parameters.
 *
 * @author Francis J. Lacoste
 */
public class Parser {

  /**
   * Service the request.
   *
   * @exception  ServletException if there is any sort of Exception
   *
   */
  public static void parseHtml
  (
   String templateFile, 
   Hashtable fields, 
   ServletOutputStream out,
   RecordParser rparser
    )
       throws IOException, ServletException
  {
    //response.setContentType("text/html");
    //ServletOutputStream out = response.getOutputStream();
    //String filename = request.getPathTranslated();

    File htmlfile = new File( templateFile );

    if(!htmlfile.isFile()) 
      {
	throw new IOException("Could not open the file "+ templateFile +",");
      } else {
	
	BufferedReader lines = 
	  new BufferedReader
	  (new InputStreamReader
	   (new FileInputStream( htmlfile )));
	
	String aline, record="";
	int pos;
	boolean recordCollect = false;
	while ( ( aline = lines.readLine() ) != null )
	  {
	    if(!recordCollect)
	      {
		pos = aline.indexOf("<record>");
		if(pos >= 0 ) // this line needs a special treatment b/of the <record> tag.
		  {
		    if(pos>0) // there must be leading characters, so let parseLine handle those.
		      parseLine(fields, aline.substring(0, pos), out);
		    
		    aline = aline.substring(pos+8); // trim the leading chars, as well as <record> .
		    recordCollect = true;
		  }
		else // no record tag, just continue.
		  Parser.parseLine(fields, aline, out);
	      }

	    if(recordCollect)
	      {
		pos = aline.indexOf("</record>");
		if(pos >= 0) // <record> tag is closed.
		  {
		    record += aline.substring(0, pos); // code between tags are captured.
		    
		    // put the record treatment here!!!

		    while( rparser.handleRecord(record, fields, out) ); 

		    out.println("<font color=red>"+record+"</font>");

		    // put the record treatment here!!!
		    
		    aline = aline.substring(pos+9);   // trim the closing record tag.
		    Parser.parseLine(fields, aline, out); // print the rest of the line.
		    recordCollect = false;
		  }
		else
		  record += "\n"+aline; // tag is not closed and start collecting
	      }
	  }
      }
  }
  
  public static void parseLine
  (Hashtable fields,
   String aline,
   ServletOutputStream out
    )
       throws IOException, ServletException
  {
    int pos = aline.indexOf("$"); // each variable must start with '$'.
    while (pos >= 0 && aline.length()>1)
      {
	if( pos != 0 ) 
	  {
	    if( aline.charAt(pos-1) == '\\' )     // escape character found!
	      aline = aline.substring(0, pos-1) + aline.substring(pos);
	    out.print( aline.substring(0, pos) ); // replace esc+$ w/ $
	    aline = aline.substring(pos);         // and print the heading part.
	    // continue w/ the rest.
	  } 
	// first char after $ must be an identifier starter usu. alphabetic chars.
	// we leave the job to Java.
	else if( Character.isUnicodeIdentifierStart(aline.charAt(1)) )
	  aline = Parser.parseVar( fields, aline ); 		                                           
	else 
	  { // if it is not a starter, then print $ and continue w/ the rest.
	    out.print( aline.substring(0, 1) );
	    aline = aline.substring(1);
	  }
	pos = aline.indexOf("$");
      }
    out.println( aline );
  }
  
  public static String parseVar
  (
   Hashtable fields, 
   String aline
    )
  {
    if(aline.length() > 1) 
      {
	int pos=1;
	String word="";
	// before coming here, we made sure that there is at least one char
	// which can make us an identifier.
	while( Character.isUnicodeIdentifierPart( aline.charAt(pos) ) ) 
	  {
	    word += aline.charAt(pos);
	    pos++;
	  }

	if( !word.equals("") )
	  {
	    String value = (String)fields.get(word);
	    aline = value + aline.substring(pos);
	  }
      }
    return aline;
  }
}
